#include "../lib/rcsid.h"

RCS_ID("$Header: svn+ssh://kcase.org/Source/svn/kcase/tags/bb/2.6/misc/bb-dump.c 745 2007-12-09 07:55:30Z kc $")

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include "../server/config.h"
#include "../server/database.h"

static void dump_everything(void);
static void dump_groups(void);
static void dump_users(void);
static void dump_rooms(void);
static void dump_room_users(void);
static void dump_messages(void);

static boolean *room_exists = NULL;
static boolean *user_exists = NULL;

int main(int argc, char *argv[])
{
    char message_base[80] = MESSAGEBASE;
    int ch;
    extern char *optarg;

    while ((ch = getopt(argc, argv, "M:")) != EOF) {
	switch (ch) {
            case 'M':
                strcpy(message_base, optarg);
                break;
	}
    }

    if (chdir(message_base) != 0) {
        perror(message_base);
        return 1;
    }

    if (!database_init(TRUE, FALSE)) {
        perror("database init failed");
        return 1;
    }
    dump_everything();
    database_shutdown();

    return 0;
}

static void dump_everything(void)
{
    int byteOrder = 0x01020304;
    fwrite((char *)&byteOrder, sizeof(byteOrder), 1, stdout);
    
    dump_users();
    dump_groups();
    dump_rooms();
    dump_room_users();
    dump_messages();

    fprintf(stderr, "done!\n");
}

static void dump_groups(void)
{
    bb_gid gid;
    
    fprintf(stderr, "Dumping %ld groups...\n", Last_gid);
    fflush(stderr);
    for (gid = 0; gid <= Last_gid; gid++) {
        DbGroup group_info;
        
	if (!get_db_group(&group_info, gid))
	    continue;
	fwrite((char *)&gid, sizeof(gid), 1, stdout);
	fwrite((char *)&group_info, sizeof(group_info), 1, stdout);
    }
    gid = -1;
    fwrite((char *)&gid, sizeof(gid), 1, stdout);
}

static void dump_users(void)
{
    bb_uid uid;

    fprintf(stderr, "Dumping %ld users...\n", Last_uid);
    fflush(stderr);
    user_exists = (boolean *) malloc((Last_uid + 1) * sizeof(boolean));
    bzero((char *)user_exists, (Last_rid + 1) * sizeof(boolean));
    for (uid = 0; uid <= Last_uid; uid++) {
        DbUser user_info;

	if (!get_db_user(&user_info, uid))
	    continue;

	user_exists[uid] = TRUE;
	fwrite((char *)&uid, sizeof(uid), 1, stdout);
	fwrite((char *)&user_info, sizeof(user_info), 1, stdout);
    }
    uid = -1;
    fwrite((char *)&uid, sizeof(uid), 1, stdout);
}

static void dump_rooms(void)
{
    static bb_rid rid;

    fprintf(stderr, "Dumping %ld rooms...\n", Last_rid);
    fflush(stderr);
    room_exists = (boolean *)malloc((Last_rid + 1) * sizeof(boolean));
    bzero((char *)room_exists, (Last_rid + 1) * sizeof(boolean));
    for (rid = 0; rid <= Last_rid; rid++) {
        DbRoom *room_info = get_db_room(rid);
        if (room_info == NULL)
	    continue;
	room_exists[rid] = TRUE;
        DbDirectory *directory = room_info->directory;
	fwrite((char *)&rid, sizeof(rid), 1, stdout);
	fwrite((char *)directory, sizeof(*directory), 1, stdout);
    }
    rid = -1;
    fwrite((char *)&rid, sizeof(rid), 1, stdout);
}

static void dump_message(bb_rid rid, int messageNumber, DbMessage *message)
{
    fwrite((char *)&messageNumber, sizeof(messageNumber), 1, stdout);
    fwrite((char *)&message->date, sizeof(message->date), 1, stdout);
    fwrite((char *)&message->expiration, sizeof(message->expiration), 1, stdout);
    fwrite((char *)&message->size, sizeof(message->size), 1, stdout);
    fwrite((char *)&message->lines, sizeof(message->lines), 1, stdout);
    fwrite((char *)&message->deleted, sizeof(message->deleted), 1, stdout);
    fwrite((char *)&message->poster_uid, sizeof(message->poster_uid), 1, stdout);
    fwrite(message->poster_nick, sizeof(message->poster_nick), 1, stdout);
    fwrite(message->poster_id, sizeof(message->poster_id), 1, stdout);
    fwrite(message->subject, sizeof(message->subject), 1, stdout);
}

static void dump_messages_in_room(bb_rid rid)
{
    DbRoom *room;
    int messageIndex, messageCount;
    
    if (!room_exists[rid])
        return;
    
    room = get_db_room(rid);
    if (room == NULL)
        return;

    messageCount = room->directory->num_msgs;
    if (messageCount == 0)
        return;

    fprintf(stderr, "%4ld ", rid);
    fflush(stderr);
    fwrite((char *)&rid, sizeof(rid), 1, stdout);
    fwrite((char *)&messageCount, sizeof(messageCount), 1, stdout);
    
    for (messageIndex = 0; messageIndex < messageCount; messageIndex++) {
        DbSummary *summary;

        summary = room->summary[messageIndex];
        dump_message(rid, summary->number, &summary->header);
    }
}

static void dump_messages(void)
{
    bb_rid rid;

    fprintf(stderr, "Dumping messages...\n");
    fflush(stderr);
    for (rid = 0; rid <= Last_rid; rid++) 
        dump_messages_in_room(rid);
    rid = -1;
    fwrite((char *)&rid, sizeof(rid), 1, stdout);
    fprintf(stderr, "\n");
}

static void dump_room_users(void)
{
    bb_uid uid;

    fprintf(stderr, "Dumping user pointers...\n");
    fflush(stderr);
    for (uid = 0; uid <= Last_uid; uid++) {
        bb_rid rid;

	if (!user_exists[uid])
	    continue;
	fprintf(stderr, "%4ld ", uid);
	fflush(stderr);
	fwrite((char *)&uid, sizeof(uid), 1, stdout);

	for (rid = 0; rid <= Last_rid; rid++) {
            DbRoomUser room_user_info;

	    if (!room_exists[rid])
		continue;
	    if (!get_db_room_user(&room_user_info, rid, uid))
		continue;

	    fwrite((char *)&rid, sizeof(rid), 1, stdout);
	    fwrite((char *)&room_user_info, sizeof(room_user_info), 1, stdout);
	}
	rid = -1;
	fwrite((char *)&rid, sizeof(rid), 1, stdout);
    }
    uid = -1;
    fwrite((char *)&uid, sizeof(uid), 1, stdout);
    fprintf(stderr, "\n");
}

static void old_code(void)
{
    fwrite((char *)&Last_gid, sizeof(Last_gid), 1, stdout);
    fwrite((char *)&Last_rid, sizeof(Last_rid), 1, stdout);
    fwrite((char *)&Last_uid, sizeof(Last_uid), 1, stdout);
}
