#include "../lib/rcsid.h"

RCS_ID("$Header: svn+ssh://kcase.org/Source/svn/kcase/tags/bb/2.6/misc/bb-load.c 770 2008-01-24 08:22:24Z kc $")

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include "../server/config.h"
#include "../server/database.h"
#include "convertlib.h"

static int load_everything(void);
static void load_users(void);
static void load_groups(void);
static void load_rooms(void);
static void load_room_users(void);
static void load_messages(void);

int main(int argc, char *argv[])
{
    char message_base[80] = "";
    int ch;
    extern char *optarg;

    while ((ch = getopt(argc, argv, "M:")) != EOF)
	switch (ch) {
	case 'M':
	    strcpy(message_base, optarg);
	    break;
	}
    if (!*message_base) {
	fprintf(stderr, "usage: %s -M messagebase [-t]\n", argv[0]);
	return 1;
    }
    if (chdir(message_base) != 0) {
	perror(message_base);
	return 1;
    }
    if (!database_init(FALSE, FALSE)) {
	perror("database init failed");
	return 1;
    }
    if (load_everything() != 0)
        return 1;

    database_shutdown();
    return 0;
}

static int load_everything(void)
{
    int test, byteOrder = 0x01020304;
    fread(&test, sizeof(test), 1, stdin);
    Convert = (test != byteOrder);
    if (Convert)
        flipl(test);

    if (test != 0x01020304) {
	fprintf(stderr, "unrecognized format\n");
	return 1;
    }

    load_users();
    load_groups();
    load_rooms();
    load_room_users();
    load_messages();
    database_changelast(Last_gid, Last_rid, Last_uid);

    fprintf(stderr, "done!\n");
    fflush(stderr);
    dump_cache();

    return 0;
}


static void load_groups(void)
{
    printf("Loading groups. . .\n");
    fflush(stdout);
    do {
        bb_gid gid;
	fread((char *)&gid, sizeof(gid), 1, stdin);
	convert_gid(&gid);
	if (gid == -1)
	    break;

        if (gid > Last_gid)
            Last_gid = gid;
        DbGroup group_info;
	fread((char *)&group_info, sizeof(group_info), 1, stdin);
	convert_group_entry(&group_info);
	if (!put_gid(group_info.name, gid))
	    perror("put_gid");
	if (!put_db_group(gid, &group_info))
	    perror("put_group_entry");
    } while (1);
}

static void load_users(void)
{
    printf("Loading users. . .\n");
    fflush(stdout);
    do {
        bb_uid uid;
	fread((char *)&uid, sizeof(uid), 1, stdin);
	convert_uid(&uid);
	if (uid == -1)
	    break;

        if (uid > Last_uid)
            Last_uid = uid;
        DbUser user_info;
	fread((char *)&user_info, sizeof(user_info), 1, stdin);
	convert_user_entry(&user_info);
	if (!put_uid(user_info.id, uid))
	    perror("put_uid");
	if (!put_db_user(uid, &user_info))
	    perror("put_user_entry");
    } while (1);
}

static void load_rooms(void)
{
    printf("Loading rooms. . .\n");
    fflush(stdout);
    do {
        bb_rid rid;
	fread((char *)&rid, sizeof(rid), 1, stdin);
	convert_rid(&rid);
	if (rid == -1)
	    break;

        DbDirectory directory;
	fread((char *)&directory, sizeof(directory), 1, stdin);
	convert_directory_entry(&directory);
        directory.num_msgs = 0;
        bzero(&directory.msg_sid, sizeof(directory.msg_sid));
	if (add_room(directory.name, rid, directory.arid) != rid) {
	    perror("add_room");
            exit(1);
        }
        DbDirectory *newDirectory = get_db_room_directory(rid);
        if (newDirectory == NULL) {
            perror("get_db_room_directory");
            exit(1);
        }
        memcpy(newDirectory, &directory, sizeof(DbDirectory));
	if (!put_db_room(rid))
	    perror("put_directory_entry");
    } while (1);
}

static void load_messages(void)
{
    printf("Loading messages. . .\n");
    fflush(stdout);
    do {
        bb_rid rid;
	fread((char *)&rid, sizeof(rid), 1, stdin);
	convert_rid(&rid);
	if (rid == -1)
	    break;
	printf("%4d ", (int)rid);
	fflush(stdout);

        DbRoom *room;
        room = get_db_room(rid);

        int messageIndex, messageCount;
	fread((char *)&messageCount, sizeof(messageCount), 1, stdin);
	flipl(messageCount);
	for (messageIndex = 0; messageIndex < messageCount; messageIndex++) {
            int messageNumber;
            DbMessage message;
	    fread((char *)&messageNumber, sizeof(messageNumber), 1, stdin);
	    fread((char *)&message.date, sizeof(message.date), 1, stdin);
	    fread((char *)&message.expiration, sizeof(message.expiration), 1, stdin);
	    fread((char *)&message.size, sizeof(message.size), 1, stdin);
	    fread((char *)&message.lines, sizeof(message.lines), 1, stdin);
	    fread((char *)&message.deleted, sizeof(message.deleted), 1, stdin);
	    fread((char *)&message.poster_uid, sizeof(message.poster_uid), 1, stdin);
	    fread(message.poster_nick, sizeof(message.poster_nick), 1, stdin);
	    fread(message.poster_id, sizeof(message.poster_id), 1, stdin);
	    fread(message.subject, sizeof(message.subject), 1, stdin);
            flipl(messageNumber);
            convert_message_entry(&message);
            put_db_message(rid, messageNumber, &message);
	}
    } while (1);
    printf("\n");
}


static void load_room_users(void)
{
    printf("Loading user pointers. . .\n");
    fflush(stdout);
    do {
        bb_uid uid;
	fread((char *)&uid, sizeof(uid), 1, stdin);
	convert_uid(&uid);
	if (uid == -1)
	    break;
	printf("%4d ", (int)uid);
	fflush(stdout);
	do {
            bb_rid rid;
	    fread((char *)&rid, sizeof(rid), 1, stdin);
	    convert_rid(&rid);
	    if (rid == -1)
		break;
            DbRoomUser room_user_info;
	    fread((char *)&room_user_info, sizeof(room_user_info), 1, stdin);
	    convert_room_user_entry(&room_user_info);
	    if (!put_db_room_user(rid, uid, &room_user_info))
		continue;
	} while (1);
    } while (1);
    printf("\n");
}
