#include <ndbm.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>

#include "../server/server.h"

typedef struct {
  char type;
  char room[ROOM_SIZE];
  long message;
} room_message;

typedef struct {
  char type;
  char room[ROOM_SIZE];
  char id[ID_SIZE];
} room_id_t;

char *classes[4] = {"no_access","read_only","read_write","moderator"};

main(argc,argv)
     int argc;
     char *argv[];
{
  datum key, content;
  DBM *dbm;

  argc--; argv++;
  if(!argc) exit(0);
  for(;argc;argc--,argv++) {
    printf("-------------------- %s --------------------\n",*argv);
    dbm = dbm_open(*argv,O_RDONLY,0);
    if(dbm == NULL) {
      continue;
    }
    for (key = dbm_firstkey(dbm);
	 key.dptr != NULL;
	 key = dbm_nextkey(dbm)) {
      content = dbm_fetch(dbm,key);
      if(content.dsize == sizeof(directory_entry)) {
	printf("directory_entry '%s' '%s' %d '%s' %d %d %d %d '%s'\n",
	       key.dptr,
	       ((directory_entry *) content.dptr)->owner,
	       ((directory_entry *) content.dptr)->creation_time,
	       classes[((directory_entry *) content.dptr)->default_class],
	       ((directory_entry *) content.dptr)->default_expiration,
	       ((directory_entry *) content.dptr)->next_message_number,
	       ((directory_entry *) content.dptr)->final_message_number,
	       ((directory_entry *) content.dptr)->first_message_number,
	       ((directory_entry *) content.dptr)->summary_line);
      } else if(content.dsize == sizeof(user_entry)) {
	printf("user_entry '%s' '%s' %d '%s' '%s' '%s' %d\n",
	       key.dptr,
	       ((user_entry *) content.dptr)->last_access_source,
	       ((user_entry *) content.dptr)->last_access_time,
	       ((user_entry *) content.dptr)->passwd,
	       ((user_entry *) content.dptr)->nick,
	       ((user_entry *) content.dptr)->default_room,
	       ((user_entry *) content.dptr)->uses);
      } else if(content.dsize == sizeof(room_user_entry)) {
	if(((room_user_entry *) content.dptr)->class > moderator) {
	  fprintf(stderr,"*** room_user_entry ('%s','%s') %d %d %d\n",
		  ((room_id_t *) key.dptr)->room,
		  ((room_id_t *) key.dptr)->id,
		  ((room_user_entry *) content.dptr)->class,
		  ((room_user_entry *) content.dptr)->last_message_read,
		  ((room_user_entry *) content.dptr)->hidden);
	} else {
	  printf("room_user_entry ('%s','%s') %s %d %d\n",
		 ((room_id_t *) key.dptr)->room,
		 ((room_id_t *) key.dptr)->id,
		 classes[((room_user_entry *) content.dptr)->class],
		 ((room_user_entry *) content.dptr)->last_message_read,
		 ((room_user_entry *) content.dptr)->hidden);
	}
      } else if(content.dsize == sizeof(message_entry)) {
	printf("message_entry ('%s',%d) %d %d %d %d '%s' '%s'\n",
	       ((room_message *) key.dptr)->room,
	       ((room_message *) key.dptr)->message,
	       ((message_entry *) content.dptr)->deleted,
	       ((message_entry *) content.dptr)->size,
	       ((message_entry *) content.dptr)->date,
	       ((message_entry *) content.dptr)->expiration,
	       ((message_entry *) content.dptr)->poster,
	       ((message_entry *) content.dptr)->subject);
      } else {
	fwrite(key.dptr,key.dsize,1,stdout);
	printf("\n%d\t",content.dsize);
	fwrite(content.dptr,content.dsize,1,stdout);
	printf("\n\n");
      }
    }
    dbm_close(dbm);
  }
}
