#include "../server/server.h"
#include <sys/time.h>

static const char *message_summary(int number, DbMessage *entry, boolean full, int width);

int main(int argc, char *argv[])
{
    extern char *optarg;
    extern int optind;
    int ch;
    char message_base[80] = MESSAGEBASE;
    char room[ROOM_SIZE];
    bb_rid rid;
    DbRoom *db_room;
    int messageIndex;

    while ((ch = getopt(argc, argv, "M:t")) != EOF)
	switch (ch) {
	case 'M':
	    strcpy(message_base, optarg);
	    break;
	}
    if (chdir(message_base) != 0) {
	perror(message_base);
	return 1;
    }
    if (!database_init(TRUE, FALSE)) {
	perror("database init failed");
	return 1;
    }
    // openlog("summary", LOG_PID | LOG_PERROR, LOG_LOCAL2);
    bzero(room, sizeof(room));
    if (optind >= argc) {
	printf("usage: %s [-M messagebase] room\n", argv[0]);
	return 1;
    }
    strcpy(room, argv[optind]);
    if ((rid = get_rid(room)) == -1) {
	fprintf(stderr, "%s: no such room\n", room);
	return 1;
    }
    setlinebuf(stdout);
    db_room = get_db_room(rid);
    if (db_room == NULL || db_room->summary == NULL) {
	fprintf(stderr, "Error retrieving summary for %s (%ld)\n", room, rid);
	return 1;
    }
    for (messageIndex=0; messageIndex < db_room->directory->num_msgs; messageIndex++) {
        DbSummary *db_summary;
        DbMessage *db_message;

        db_summary = db_room->summary[messageIndex];
        db_message = &(db_summary->header);
	printf("%s\n", message_summary(db_summary->number, db_message, TRUE, 80));
    }
    database_shutdown();
    return 0;
}

char *Date(time_t date)
{
    char *d;

    d = ctime(&date);
    *(d + 24) = '\0';
    return d;
}

static const char *message_summary(int number, DbMessage *entry, boolean full, int width)
{
    static char line[BUFFER_SIZE];

    if (full) {
	sprintf(line, "%c %4d %12.12s %4d [%10.10s] %s",
		entry->deleted ? '*' : ' ',
		number,
		Date(entry->date) + 4,
		entry->lines,
		entry->poster_id,
		entry->subject);
    } else {
	sprintf(line, "(%12.12s) %4d [%10.10s] %s",
		Date(entry->date) + 4,
		entry->lines,
		entry->poster_id,
		entry->subject);
    }
    line[width - 1] = '\0';
    return line;
}
